/*
 * This file is part of the libopencm3 project.
 *
 * Copyright (C) 2009 Uwe Hermann <uwe@hermann-uwe.de>
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <libopencm3/stm32/rcc.h>
#include <libopencm3/stm32/gpio.h>
#include <libopencm3/stm32/usart.h>
#include <libopencm3/stm32/i2c.h>
#include <libopencm3/cm3/nvic.h>
#include <libopencm3/cm3/systick.h>

#include <libcommon/iob.h>
#include <libcommon/microvty.h>

#include <stdio.h>

#include "attenuator.h"
#include "misc.h"

extern const struct attenuator_cfg board_att_cfg;
extern struct attenuator_state *board_att_st[];

volatile uint32_t jiffies;

void sys_tick_handler(void)
{
	jiffies++;
}

/* busy wait specified number of ms */
void msleep(uint32_t delay)
{
	uint32_t wake = jiffies + delay;
	do {
	} while (wake > jiffies);
}


static void clock_setup(void)
{
	rcc_clock_setup_in_hsi_out_48mhz();
	/* HCLK=48MHz; ADC=6MHz; APB1=24MHz; APB2=48MHz; USB=48MHz */

	/* tick rate of 1ms */
	systick_set_reload(48000);
	systick_set_clocksource(STK_CSR_CLKSOURCE_AHB);
	systick_counter_enable();
	systick_interrupt_enable();

	/* Enable GPIOC clock. */
	rcc_periph_clock_enable(RCC_GPIOA);
	rcc_periph_clock_enable(RCC_GPIOB);
	rcc_periph_clock_enable(RCC_GPIOC);

	/* Enable clocks for USARTs */
	rcc_periph_clock_enable(RCC_USART1);
	rcc_periph_clock_enable(RCC_USART2);
}

static void i2c_setup(void)
{
	rcc_periph_clock_enable(RCC_I2C1);

	gpio_set_mode(GPIOB, GPIO_MODE_OUTPUT_50_MHZ,
		      GPIO_CNF_OUTPUT_ALTFN_OPENDRAIN,
		      GPIO_I2C1_SCL | GPIO_I2C1_SDA);

	/* disable before making config changes */
	i2c_peripheral_disable(I2C1);

	/* APB1 runs at 24 MHz */
	i2c_set_clock_frequency(I2C1, I2C_CR2_FREQ_24MHZ);

	/* 400 kHz fast mode I2C */
	//i2c_set_fast_mode(I2C1);

	//...

	i2c_peripheral_enable(I2C1);
}

static void usart_setup(void)
{
	/* USART1: connected to debug header */

	/* Setup GPIO pin GPIO_USART1_TX. */
	gpio_set_mode(GPIOA, GPIO_MODE_OUTPUT_50_MHZ,
		      GPIO_CNF_OUTPUT_ALTFN_PUSHPULL, GPIO_USART1_TX);
	/* Setup UART parameters. */
	usart_set_baudrate(USART1, 115200);
	usart_set_databits(USART1, 8);
	usart_set_stopbits(USART1, USART_STOPBITS_1);
	usart_set_mode(USART1, USART_MODE_TX_RX);
	usart_set_parity(USART1, USART_PARITY_NONE);
	usart_set_flow_control(USART1, USART_FLOWCONTROL_NONE);
	/* Finally enable the USART. */
	usart_enable(USART1);

	/* USART2: connected to 2.5mm stereo jack */

	/* Setup GPIO pin GPIO_USART2_TX. */
	gpio_set_mode(GPIOA, GPIO_MODE_OUTPUT_50_MHZ,
		      GPIO_CNF_OUTPUT_ALTFN_PUSHPULL, GPIO_USART2_TX);
	/* Setup UART parameters. */
	usart_set_baudrate(USART2, 115200);
	usart_set_databits(USART2, 8);
	usart_set_stopbits(USART2, USART_STOPBITS_1);
	usart_set_mode(USART2, USART_MODE_TX_RX);
	usart_set_parity(USART2, USART_PARITY_NONE);
	usart_set_flow_control(USART2, USART_FLOWCONTROL_NONE);
	/* Finally enable the USART. */
	usart_enable(USART2);
}

static void gpio_setup(void)
{
	/* Set GPIO15 (in GPIO port B) to 'output push-pull'. */
	gpio_set_mode(GPIOB, GPIO_MODE_OUTPUT_2_MHZ,
		      GPIO_CNF_OUTPUT_PUSHPULL, GPIO15);
}

int main(void)
{
	clock_setup();
	gpio_setup();
	usart_setup();
	iob_init(USART2);
	i2c_setup();
	attenuator_init(&board_att_cfg, board_att_st);
	microvty_init("rfdsat4ch> ");

	printf("Welcome to RFDSATT\r\n");
	fprintf(stderr, "Welcome to stderr\r\n");

	/* Blink the LED (PB15) on the board with every transmitted byte. */
	while (1) {
		int i;
		gpio_toggle(GPIOB, GPIO15);	/* LED on/off */
		//printf("Hello world\r\n");
		microvty_try_recv();
		for (i = 0; i < 800000; i++)	/* Wait a bit. */
			__asm__("nop");
	}

	return 0;
}
